# -*- coding: utf-8 -*-
#
# Copyright (C) Saeed Rasooli <saeed.gnu@gmail.com>
# Copyright (C) 2007 Mehdi Bayazee <Bayazee@Gmail.com>
# Copyright (C) 2001 Roozbeh Pournader <roozbeh@sharif.edu>
# Copyright (C) 2001 Mohammad Toossi <mohammad@bamdad.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/gpl.txt>.
# Also avalable in /usr/share/common-licenses/GPL on Debian systems
# or /usr/share/licenses/common/GPL3/license.txt on ArchLinux

## Iranian (Jalali) calendar:
## http://en.wikipedia.org/wiki/Iranian_calendar

import os
import sys
here = os.path.dirname(os.path.realpath(__file__))
sys.path.append(here)

name = 'jalali'
desc = 'Jalali'
origLang = 'fa'

monthNameMode = 0
jalaliAlg = 1
options = (
    (
        'monthNameMode',
        list,
        'Jalali Month Names',
        ('Iranian', 'Kurdish', 'Dari', 'Pashto'),
    ),
    (
        'jalaliAlg',
        list,
        'Jalali Calculation Algorithm',
        ('33 year algorithm', '2820 year algorithm'),
    ),
)


monthNameVars = (
    (
        ('Farvardin','Ordibehesht','Khordad','Teer','Mordad','Shahrivar',
         'Mehr','Aban','Azar','Dey','Bahman','Esfand'),
        ('Far', 'Ord', 'Khr', 'Tir', 'Mor', 'Shr',
         'Meh', 'Abn', 'Azr', 'Dey', 'Bah', 'Esf'),
    ),
    (
        ('Xakelêwe','Gullan','Cozerdan','Pûşper','Gelawêj','Xermanan',
         'Rezber','Gelarêzan','Sermawez','Befranbar','Rêbendan','Reşeme'),
    ),
    (
        ('Hamal','Sawr','Jawzā','Saratān','Asad','Sonbola',
         'Mizān','Aqrab','Qaws','Jadi','Dalvæ','Hūt'),
    ),
    (
        ('Wray','Ǧwayay','Ǧbargolay','Čungāx̌','Zmaray','Waǵay',
         'Təla','Laṛam','Līndəi','Marǧūmay','Salwāǧa','Kab'),
    ),
)

#        ('','','','','','',
#         '','','','','','')


getMonthName = lambda m, y=None: monthNameVars[monthNameMode][0][m-1]

def getMonthNameAb(m, y=None):
    v = monthNameVars[monthNameMode]
    try:
        l = v[1]
    except IndexError:
        l = v[0]
    return l[m-1]



getMonthsInYear = lambda y: 12


epoch = 1948321
minMonthLen = 29
maxMonthLen = 31
avgYearLen = 365.2425 ## FIXME

GREGORIAN_EPOCH = 1721426
monthLen = (31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 29)


import os
#from scal2.path import sysConfDir, confDir
#from scal2.utils import iceil
#from scal2.utils import myRaise
from math import ceil
iceil = lambda x: int(ceil(x))

'''
## Here load user options(jalaliAlg) from file
sysConfPath = '%s/%s.conf'%(sysConfDir, name)
if os.path.isfile(sysConfPath):
    try:
        exec(open(sysConfPath).read())
    except:
        myRaise(__file__)

confPath = '%s/%s.conf'%(confDir, name)
if os.path.isfile(confPath):
    try:
        exec(open(confPath).read())
    except:
        myRaise(__file__)



def save():## Here save user options to file
    text = ''
    text += 'monthNameMode=%s\n'%monthNameMode
    text += 'jalaliAlg=%s\n'%jalaliAlg
    open(confPath, 'w').write(text)
'''


def isLeap(year):
    "isLeap: Is a given year a leap year in the Jalali calendar ?"
    if jalaliAlg==1:## 2820-years
        return (( (year - 473 - (year>0)) % 2820) * 682) % 2816 < 682
    elif jalaliAlg==0:## 33-years
        jy = year - 979
        gdays = ( 365*jy + (jy//33)*8 + (jy%33+3)//4 + 79 ) % 146097
        ## 36525 = 365*100 + 100//4
        if gdays >= 36525:
            gdays = (gdays-1) % 36524 + 1
            if gdays < 366:
                return False
        if gdays % 1461 >= 366:
            return False
        return True

    else:
        raise RuntimeError('bad option jalaliAlg=%s'%jalaliAlg)

def to_jd(year, month, day):
    "TO_JD: Determine Julian day from Jalali date"
    if jalaliAlg==1:## 2820-years
        epbase = year - 474 if year>=0 else 473
        epyear = 474 + epbase % 2820
        return day + \
            (month-1) * 30 + min(6, month-1) + \
            (epyear * 682 - 110) // 2816 + \
            (epyear - 1) * 365 + \
            epbase // 2820 * 1029983 + \
            epoch - 1
    elif jalaliAlg==0:## 33-years
        y2 = year - 979
        jdays = 365*y2 + y2//33 * 8 + (y2%33+3)//4
        for i in range(month-1):
            jdays += monthLen[i]
        jdays += (day-1)
        return jdays + 584101 + GREGORIAN_EPOCH
    else:
        raise RuntimeError('bad option jalaliAlg=%s'%jalaliAlg)

def jd_to(jd):
    "JD_TO_JALALI: Calculate Jalali date from Julian day"
    if jalaliAlg==1:## 2820-years
        cycle, cyear = divmod(jd - to_jd(475, 1, 1), 1029983)
        if cyear == 1029982 :
            ycycle = 2820
        else:
            aux1, aux2 = divmod(cyear, 366)
            ycycle = (2134*aux1 + 2816*aux2 + 2815) // 1028522 + aux1 + 1
        year = 2820*cycle + ycycle + 474
        if year <= 0 :
            year -= 1
        yday = jd - to_jd(year, 1, 1) + 1
        if yday <= 186:
            month = iceil(yday // 31)
        else:
            month = 6 + iceil((yday-186) // 30)
        day = int(jd - to_jd(year, month, 1)) + 1
        if day > 31:
            day -= 31
            if month==12:
                month = 1
                year += 1
            else:
                month += 1
    elif jalaliAlg==0:## 33-years
        jdays = int(jd - GREGORIAN_EPOCH - 584101)
        ## -(1600*365 + 1600//4 - 1600//100 + 1600//400) + 365    -79 +1== -584101
        #print('jdays =',jdays)
        j_np = jdays // 12053
        jdays %= 12053
        year = 979 + 33*j_np + 4*(jdays//1461)
        jdays %= 1461
        if jdays >= 366:
            year += (jdays-1) // 365
            jdays = (jdays-1) % 365
        month = 12
        for i in range(11):
            if jdays >= monthLen[i]:
                jdays -= monthLen[i]
            else:
                month = i+1
                break
        day = jdays+1
    else:
        raise RuntimeError('bad option jalaliAlg=%s'%jalaliAlg)
    return year, month, day


## Normal: esfand = 29 days
## Leap: esfand = 30 days

def getMonthLen(year, month):
    if month==12:
        return 29 + isLeap(year)
    else:
        return monthLen[month-1]




from pyramid.config import Configurator
from pyramid.httpexceptions import HTTPNotFound
from sqlalchemy import engine_from_config
from pyramid.authentication import AuthTktAuthenticationPolicy
from pyramid.security import authenticated_userid
from pyramid.authorization import ACLAuthorizationPolicy


from .models import (
    DBSession,
    Base,
    )
from .security import groupfinder

def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)
    Base.metadata.bind = engine

    authn_policy = AuthTktAuthenticationPolicy('fguia9yfods', callback=groupfinder, hashalg='sha512')
    authz_policy = ACLAuthorizationPolicy()

    config = Configurator(settings=settings, root_factory='.security.RootFactory')
    config.set_authentication_policy(authn_policy)
    config.set_authorization_policy(authz_policy)

    config.include('pyramid_chameleon')
    config.include('pyramid_mako')
    config.include('pyramid_tm')

    config.add_static_view('static', 'static', cache_max_age=3600)
    config.add_route('home', '/')

    config.add_route('course_add', '/add/course')
    config.add_route('question_add', '/add/question')
    config.add_route('question_group_add', '/add/question_group')

    config.add_route('course_list','/course/')
    config.add_route('course_page','/course/{course_title}')

    config.add_route('exam_list','/exam')
    config.add_route('exam_page','/exam/{exam_id}')

    config.add_route('certificate','/certificate/{exam_id}')


    config.add_route('quiz_start','/quiz/{quiz_id}')
    config.add_route('quiz_finish','/quiz/{quiz_id}/finish')
    config.add_route('quiz_result','/quiz/{quiz_id}/result')
    config.add_route('quiz_question','/quiz/{quiz_id}/{question_id}')

    config.add_route('login','/login')
    config.add_route('logout','/logout')
    config.add_route('signup','/signup')
    config.add_route('profile', '/profile/{login_name}')
    config.add_route('payment', '/payment')
    config.add_route('score_exam_list', '/score')
    config.add_route('score_exam_page', '/score/{exam_id}')
    # config.add_notfound_view(append_slash=True)

    config.scan()

    return config.make_wsgi_app()

mysettings = {'pyramid.default_locale_name': 'fa', 'pyramid.debug_authorization': 'false', 'pyramid.debug_routematch': 'false',
            'pyramid.reload_templates': 'false', 'sqlalchemy.url': 'mysql+pymysql://root:1@localhost/alchemy?charset=utf8&use_unicode=1',
            'pyramid.debug_notfound': 'false'}



__author__ = 'hamid'
import os
import sys
here = os.path.dirname(os.path.realpath(__file__))
sys.path.append(here)
from vame_ir import jalali
import datetime
d = datetime.date(2001,12,1)
a = jalali.to_jd(1393,7,16)
import time
t = time.time()
t = 1412807842 / (3600 * 24)
print(t)
print(jalali.jd_to(t))
import datetime
import _datetime
datetime.datetime
#testfdg d sgf gf
# jalali.



__author__ = 'hamid'
from .models import (
    DBSession,
    Group,
    User,
)

from pyramid.security import (
    Allow,
    Everyone,
)


def groupfinder(userid, request):

    user = DBSession.query(User).filter_by(login_name=userid).one()
    return [group.title for group in user.groups]


# TODO: try to add Group object to RootFactory
# print(DBSession.query(Group).filter_by(id=4).one())

class RootFactory(object):
    __acl__ = [(Allow, Everyone, 'view'),
               (Allow, 'group:students', 'student'),
               (Allow, 'group:editors', ('edit','student')),
               (Allow, 'group:admins', ('admin','edit','student'))]

    def __init__(self, request):
        pass



import os
import sys
import transaction

from sqlalchemy import engine_from_config

from pyramid.paster import (
    get_appsettings,
    setup_logging,
    )

from pyramid.scripts.common import parse_vars

from ..models import (
    DBSession,
    User,
    Base,
    Group,
    Question_Group,
    Question,
    Course,
    Exam,
    Answer,

    )


def usage(argv):
    cmd = os.path.basename(argv[0])
    print('usage: %s <config_uri> [var=value]\n'
          '(example: "%s development.ini")' % (cmd, cmd))
    sys.exit(1)


def main(argv=sys.argv):
    if len(argv) < 2:
        usage(argv)
    config_uri = argv[1]
    options = parse_vars(argv[2:])
    setup_logging(config_uri)
    settings = get_appsettings(config_uri, options=options)
    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)
    Base.metadata.create_all(engine)
    with transaction.manager:
        users = DBSession.query(User).all()
        if not users:
        # user1 = User(login_name='root', password='qdbwfnegm', first_name='Super User', last_name = 'Super User', email = 's.h.mahdavi@chmail.ir',
            user1 = User(login_name='root', password='1', first_name=' ', last_name = ' کاربر ارشد', email = 's.h.mahdavi@chmail.ir',
                     phone = '09106853582', mobile = '09106853582')
            user1.account = 200000000
            group1 = Group("group:admins")
            user1.groups.append(group1)
        # question_groups = DBSession.query(Question_Group).all()
        # if not question_groups:
            question_group1 = Question_Group(title='گروه اول')
            question_group2 = Question_Group(title='گروه دوم')

        # questions = DBSession.query(Question).all()
        # if not questions:
            question1 = Question(title='پایتخت ایران کجا است', time=30, comment=' ', correct_answer='تهران')
            question2 = Question(title='سعدی کجا دفن است', time=30, comment=' ', correct_answer='شیراز')
            question1.groups.append(question_group1)
            question1.groups.append(question_group2)
            question2.groups.append(question_group2)

            course1 = Course(title='first Course',comment='first course commet',time=300 ,price=300000,max_question=5)
            course2 = Course(title='second Course',comment='second course commet',time=400 ,price=200000,max_question=5)
            course1.questions.append(question1)
            course1.questions.append(question2)
            course2.questions.append(question2)

            group2 = Group("group:students")
            group3 = Group("group:editors")
            DBSession.add(group2)
            DBSession.add(group3)
            DBSession.add(question_group1)
            DBSession.add(question_group2)
            DBSession.add(question2)
            DBSession.add(question1)
            DBSession.add(course1)
            DBSession.add(user1)



from pyramid.security import (
    Allow,
    Everyone,
)
from sqlalchemy import (
    Column,
    Index,
    Integer,
    Text,
    String,
    Table,
    ForeignKey,
    )

from sqlalchemy.ext.declarative import declarative_base

from sqlalchemy.orm import (
    scoped_session,
    sessionmaker,
    relationship,
    backref,
    )

from zope.sqlalchemy import ZopeTransactionExtension

DBSession = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
Base = declarative_base()

user_2_group_map = Table('user_2_group_map', Base.metadata,
                       Column('user_id', Integer, ForeignKey('users.id')),
                       Column('group_id', Integer, ForeignKey('groups.id'))
)




class User(Base):
    __tablename__ = 'users'
    id = Column(Integer, primary_key=True)
    login_name = Column(String(50))
    password = Column(String(50),nullable=False)
    first_name = Column(String(50),nullable=False)
    last_name = Column(String(50),nullable=False)
    email = Column(String(50),nullable=False)
    phone = Column(String(50),nullable=False)
    mobile = Column(String(50),nullable=False)
    payment = Column(Integer)
    account = Column(Integer)
    blocked = Column(Integer) # 0 for no - null or -1 for not activated yet - bigger than 1 for yes

    groups = relationship('Group',
                          secondary=user_2_group_map,
                          backref='users'
                          )

    exams = relationship('Exam',backref='user')

    def __init__(self, login_name, password, first_name='temp', last_name = 'temp', email = 'temp',
                 phone = '0123456', mobile = '0123456'):
        self.login_name = login_name
        self.password = password
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.phone = phone
        self.mobile = mobile
        self.blocked = -1
        self.payment = 0
        self.account = 0
        #self.account

    def __repr__(self):
        return "<User('{0}','{1}','{2}')>".format(self.login_name, self.first_name, self.last_name)


class Group(Base):
    __tablename__ = 'groups'

    id = Column(Integer, primary_key=True)
    title = Column(String(50))

    def __init__(self, title=''):
        self.title = title

    def __repr__(self):
        return "<Group('{0}')>".format(self.title)



question_2_course_map = Table('question_2_course_map', Base.metadata,
                       Column('question_id', Integer, ForeignKey('questions.id')),
                       Column('course_id', Integer, ForeignKey('courses.id'))

)

#Course use as a framework for exam so it have one to many ralation with exam
#it also have a many to many relation with user
class Course(Base):
    __tablename__ = 'courses'
    id = Column(Integer, primary_key=True)
    title = Column(String(50), unique=True)
    comment = Column(Text)
    time = Column(Integer)
    price = Column(Integer)
    max_question = Column(Integer)
    min_score = Column(Integer)


    exams = relationship('Exam',backref='course')

    def __init__(self, title = '', comment = '', time = '', price = '', max_question = '', min_score = 50):
        self.title = title
        self.comment = comment
        self.time = time
        self.price = price
        self.max_question = max_question
        self.min_score = min_score


#this have a many to one ralation with user exam will be create when user want to star exam of cource
class Exam(Base):
    __tablename__ = 'exams'
    id = Column(Integer, primary_key=True)
    title = Column(String(50))
    comment = Column(Text)
    # submit_time = Column(Integer)

    submit_time = Column(Integer)
    status = Column(Integer)
    max_question = Column(Integer)

    # 0 registered not started -
    #  1 registerid started not finished
    #  2 registerid started finished not resulted
    #  3 not successd
    #  4 successd
    start_time = Column(Integer)
    payment = Column(Integer) #True or False
    answers = relationship('Answer', backref='exam')
     # = relationship('Answer',backref='question')

    user_id = Column(Integer,ForeignKey('users.id'))
    course_id = Column(Integer,ForeignKey('courses.id'))


    def __init__(self, user=None, course=None, comment=''):
        if user:
            self.user = user
        if course:
            self.course = course
            self.time = course.time
            self.max_question = course.max_question
        import time
        self.status = 0
        self.start_time = 0
        self.comment = comment
        self.submit_time = int(time.time())


    def status_finder(self):
        if self.status == 0:
            return 'ثبت نام شده'
        elif self.status == 1:
            return 'آزمون شروع شده'
        elif self.status == 2:
            return 'جواب آزمون مشخص نشده'
        elif self.status == 3:
            return 'رد شده'
        elif self.status == 4:
            return 'قبول'
        else:
            return 'نا مشخص'






#this have a many to many relation with exam 


question_2_question_group_map = Table('question_2_question_group_map', Base.metadata,
                       Column('question_id', Integer, ForeignKey('questions.id')),
                       Column('group_id', Integer, ForeignKey('question_groups.id'))

)

class Question(Base):
    __tablename__ = 'questions'
    id = Column(Integer, primary_key=True)
    type = Column(String(50))
    title = Column(String(50))
    time = Column(Integer)
    comment = Column(String(50))
    correct_answer = Column(String(50))
    user_answer = Column(String(50))

    groups = relationship('Question_Group',
                          secondary=question_2_question_group_map,
                          backref='questions'
                          )

    courses = relationship('Course',
                          secondary=question_2_course_map,
                          backref='questions'
                          )

    answers = relationship('Answer',backref='question')

    def __init__(self, type='', title='', time='', comment='', correct_answer='', groups=[]):
        self.type = type
        self.title = title
        self.time = time
        self.comment = comment
        self.correct_answer = correct_answer
        for group in groups:
            if group not in self.groups:
                self.groups.append(group)


    def __repr__(self):
        return "<Question('{0}')>".format(self.title)


class Question_Group(Base):
    __tablename__ = 'question_groups'

    id = Column(Integer, primary_key=True)
    title = Column(String(50))

    def __init__(self, title=''):
        self.title = title

    def __repr__(self):
        return "<Question_Group('{0}')>".format(self.title)


class Answer(Base):
    __tablename__ = 'answers'

    id = Column(Integer, primary_key=True)
    value = Column(Text)
    score = Column(Integer)


    exam_id = Column(Integer, ForeignKey('exams.id'))
    question_id = Column(Integer, ForeignKey('questions.id'))

    def __init__(self, question):
        self.question = question
        self.value = ''
        # self.value = value

    def find_score(self):
        if self.score == 0:
            return 'تصحیح نشده'
        elif self.score == 1:
            return 'پاسخ غلط'
        elif self.score == 2:
            return 'پاسخ صحیح'
        else:
            return 'نا مشخص'









Index('user_index', User.login_name, unique=True, mysql_length=50)



from pyramid.response import Response
from pyramid.view import (
    view_config,
    forbidden_view_config,
    )

from pyramid.security import (
    remember,
    forget,
    )

from pyramid.httpexceptions import (
    HTTPFound,
    HTTPForbidden,
    )

from sqlalchemy.exc import DBAPIError

from .models import (
    DBSession,
    User,
    Group,
    Question_Group,
    Question,
    Course,
    Exam,
    Answer,
    )

from .jalali import to_jd , jd_to
import transaction
import time


@view_config(route_name='home', renderer='templates/home.pt', permission='view')
def home(request):

    try:
        user = DBSession.query(User).filter(User.login_name == request.authenticated_userid).first()
    except DBAPIError:
        return Response('<html><body>connection error</body></html>', content_type='text/html', status_int=500)
    return {'user':user, 'project': 'vame course'}


@view_config(route_name='course_list', renderer='templates/course_list.pt')
def course_list(request):

    user = DBSession.query(User).filter_by(login_name=request.authenticated_userid).first()

    message = ''
    courses = DBSession.query(Course).all()


    # print(user)
    return {'message':message,'courses':courses, 'user':user}



@view_config(route_name='course_page', renderer='templates/course_page.pt',permission='student')
def course_page(request):
    message = ''

    user = DBSession.query(User).filter_by(login_name=request.authenticated_userid).one()
    course_title = request.matchdict['course_title']
    try:
        course = DBSession.query(Course).filter_by(title=course_title).one()
    except:
        message = 'course not found'
        return {'message':message, 'user':user}

    exams = [exam for exam in user.exams if exam in course.exams]
    can_register = True
    for exam in exams:
        if exam.status != 3: # Not success
            can_register = False


    # print(exams)

    if 'submit' in request.POST and can_register:
        agreement = request.POST.get('agreement')
        if agreement == "agree":


            if user.account > course.price:
                user.account -= course.price
                user.payment += course.price
                if not user.account < 0:


                    exam = Exam(user=user,course=course)
                    # exam.status = 0
                    #  0 registered not started -
                    #  1 registered started not finished
                    #  2 registered started finished not resulted
                    #  3 not success
                    #  4 success


                    # user = DBSession.query(User).filter_by(login_name=request.authenticated_userid).one()
                    # course_title = request.matchdict['course_title']
                    #
                    # course = DBSession.query(Course).filter_by(title=course_title).one()

                    DBSession.add(exam)
                    # exams = [exam for exam in user.exams if exam in course.exams]
                    exams = DBSession.query(Exam).filter_by(user_id=user.id,course_id = course.id)

                    # exams.append(exam)
                    can_register = False

                    message = 'شما در این دوره ثبت نام شدید'

                else:
                    user.account += course.price
                    user.payment -= course.price
                    message = 'اعتبار شما برای شرکت در این دوره کافی نیست. نسبت به افزایش اعتبار خود اقدام کنید'
            else:
                message = 'اعتبار شما برای شرکت در این دوره کافی نیست. نسبت به افزایش اعتبار خود اقدام کنید'

        else:
            message = 'برای شرکت در دوره باید با شرایط شرکت در دوره موافقت کنید'

    return {'message':message,'course':course, 'user':user, 'exams': exams, 'can_register': can_register}




@view_config(route_name='exam_list', renderer='templates/exam_list.pt',permission='student')
def exam_list(request):
    user = DBSession.query(User).filter_by(login_name=request.authenticated_userid).one()
    exams = DBSession.query(Exam).filter_by(user_id=user.id)
    message = ''

    return {'exams':exams,'user':user,'message':message}

@view_config(route_name='exam_page', renderer='templates/exam_page.pt',permission='student')
def exam_page(request):
    user = DBSession.query(User).filter_by(login_name=request.authenticated_userid).one()
    exam_id = request.matchdict['exam_id']
    exam = DBSession.query(Exam).filter_by(id=exam_id).first()
    message = ''


    if not exam in user.exams:
        # return HTTPFound(location=request.route_url('exam_list'))
        return HTTPForbidden()


    #######################################
    elif exam.status == 0: #  0 registered not started - in this case exam question will collected
        message = 'شما هنوز آزمون این دوره را شروع نکرده اید.'
        if 'exam.start' in request.POST:
            import random
            course = exam.course
            temporary_questions = course.questions[:]
            q_number = len(temporary_questions)
            if q_number > course.max_question:
                q_number = course.max_question

            for i in range(q_number):
                q = random.choice(temporary_questions)
                exam.answers.append(Answer(q))
                temporary_questions.remove(q)
            for a in exam.answers:
                DBSession.add(a)
            exam.status = 1
            import time
            exam.start_time = time.time()
            DBSession.add(exam)
            # exam_id = request.matchdict['exam_id']
            exam = DBSession.query(Exam).filter_by(id=exam_id).first()

            message = 'شما در حال آزمون دادن هستید'
            # transaction.commit()


    elif exam.status == 1: #  1 registered started not finished
        message = 'شما در حال آزمون دادن هستید'
        # print('salam')
        if 'exam.finish' in request.POST:
            answers = []
            for a in exam.answers:
                # my_id = a.id
                # print(my_id)
                a.value = request.POST.get(str(a.id), '')
                # print('this is: #########')
                # answers.append(an)
                # print(an)
            if request.POST.get('scored', ''):
                exam.status = 2
                message = 'جواب های شما به آزمون دریافت شد. نتیجه پس از تصحیح اوراق در سایت قرار خواهد گرفت'
            else:
                message = 'جواب های شما ذخیره شد ولی آزمون شما همچنان ادامه دارد'
            DBSession.add(exam)




    elif exam.status == 2: #  2 registered started finished not resulted
        message = 'جواب های شما به آزمون دریافت شد. نتیجه پس از تصحیح اوراق در سایت قرار خواهد گرفت'

    import time
    counter_time = int(exam.course.time - (time.time() - exam.start_time))
    print(counter_time)
    return {'exam':exam, 'message':message, 'user':user, 'counter_time':counter_time}




@view_config(route_name='quiz_start', renderer='templates/quiz_start.pt')
def quiz_start(request):
    return {'quiz': request.matchdict['quiz_id']}


@view_config(route_name='quiz_question', renderer='templates/quiz_question.pt')
def quiz_question(request):
    return {'quiz': request.matchdict['quiz_id'],'question':request.matchdict['question_id']}


@view_config(route_name='quiz_finish', renderer='templates/quiz_finish.pt')
def quiz_finish(request):
    return {'quiz': request.matchdict['quiz_id']}


@view_config(route_name='quiz_result', renderer='templates/quiz_result.pt')
def quiz_result(request):
    return {'quiz': request.matchdict['quiz_id']}


@view_config(route_name='course_add', renderer='templates/course_add.pt', permission='admin')
def course_add(request):
    message = ''
    questions = DBSession.query(Question).all()
    if 'submit' in request.POST:
        question_ids = request.POST.getall('questions')
        title = request.POST.get('title', '')
        comment = request.POST.get('comment', '')
        time = request.POST.get('time', '')
        max_question = request.POST.get('max_question', '')
        min_score = request.POST.get('min_score', '')
        price = request.POST.get('price', '')

        course = Course(title=title,comment=comment,time=time,price=price,max_question=max_question, min_score=min_score)
        for q in questions:
            if str(q.id) in question_ids:
                course.questions.append(q)
        DBSession.add(course)
        message = 'دوره جدید با موفقیت اضافه شد'




    return {'message':message,'questions':questions}

@view_config(route_name='question_add', renderer='templates/question_add.pt', permission='admin')
def question_add(request):
    message = ''
    groups = DBSession.query(Question_Group).all()
    if 'submit' in request.POST:
        question_group_ids = request.POST.getall('groups')
        title = request.POST.get('title', '')
        comment = request.POST.get('comment', '')
        time = request.POST.get('time', '')
        correct_answer = request.POST.get('correct_answer', '')
        # print(question_groups, title,time, correct_answer, comment)
        question = Question(title=title, time=time, comment=comment, correct_answer=correct_answer)
        # question_group_ids = set(question_group_ids)
        for g in groups:
            if str(g.id) in question_group_ids:
                question.groups.append(g)
        DBSession.add(question)
        message = 'سوال جدید با موفقیت اضافه شد'

    return {'message':message, 'groups':groups}


@view_config(route_name='question_group_add', renderer='templates/question_group_add.pt', permission='admin')
def question_group_add(request):
    message = ''
    if 'submit' in request.POST:
        name = request.POST.get('name', '')
        same_groups=DBSession.query(Question_Group).filter_by(title=name).all()
        if not same_groups:
            group = Question_Group(title=name)
            DBSession.add(group)
            message = "گروه جدید سولات با موفقیت اضافه شد"


    return {'message':message}


@view_config(route_name='login', renderer='templates/login.pt')
@forbidden_view_config(renderer='templates/login.pt')
def login(request):

    login_url = request.route_url('login')
    referrer = request.url
    if referrer == login_url:
        referrer = request.route_url('home')

    message = ''

    if 'submit' in request.POST:
        login_name = request.POST.get('login_name', '')
        password = request.POST.get('password', '')

        try:
            user = DBSession.query(User).filter_by(login_name=login_name,password=password).one()
            headers = remember(request, login_name)
            message = 'user logged in:{0}'.format(login_name)
            return HTTPFound(location=referrer, headers=headers)
        # try:
        #     user = DBSession.query(User).filter_by(login_name=login_name,password=password).one()
        #     print(user)
        #     headers = remember(request, login_name)
        #     message = 'user logged in:{0}'.format(user.first_name)
        #     return HTTPFound(location=referrer, headers=headers)
        except:
            message = 'کاربر پیدا نشد'

    return {'user':request.authenticated_userid, 'message':message}

@view_config(route_name='logout', renderer='templates/logout.pt')
def logout(request):
    headers = forget(request)
    return HTTPFound(location=request.route_url('home'), headers=headers)


@view_config(route_name='signup', renderer='templates/signup.pt')
def signup(request):
    message = ''
    #groups = DBSession.query(Group).all()
    if 'submit' in request.POST:
        same_users=DBSession.query(User).filter_by(login_name=request.POST.get('login_name', '')).all()
        if not same_users:
            user = User(login_name=request.POST.get('login_name', ''),
                        first_name=request.POST.get('first_name', ''),
                        last_name=request.POST.get('last_name', ''),
                        password=request.POST.get('password', ''),
                        email=request.POST.get('email', ''),
                        phone = request.POST.get('phone', ''),
                        mobile = request.POST.get('mobile', ''),)

            student_group = DBSession.query(Group).filter_by(title='group:students').one()

            user.groups.append(student_group)

            DBSession.add(user)
            message ='کاربر ثبت شد'
        else:
            message ='امکان ثبت کاربر وجود ندارد'




    return {
        'login_name':request.POST.get('login_name', ''),
        'first_name':request.POST.get('first_name', ''),
        'last_name':request.POST.get('last_name', ''),
        'password':request.POST.get('password', ''),
        'email':request.POST.get('email', ''),
        'phone':request.POST.get('phone', ''),
        'mobile':request.POST.get('mobile', ''),
        'message':message
            }

@view_config(route_name='profile', renderer='templates/profile.pt', permission='student')
def profile(request):
    # this page is profile of user
    return {}


@view_config(route_name='certificate', renderer='templates/certificate.pt', permission='view')
def certificate(request):
    # exam = request.matchdict['exam_id']
    # course = request.matchdict['course_title']
    exam = DBSession.query(Exam).filter_by(id=request.matchdict['exam_id'], status=4).first()
    # this page is profile of user

    return {'exam':exam}


@view_config(route_name='payment', renderer='templates/payment.pt', permission='student')
def payment(request):
    user = DBSession.query(User).filter_by(login_name=request.authenticated_userid).one()
    message = ''
    print('test1')
    if 'submit' in request.POST:
        print('test')
        amount = int(request.POST.get('amount',0))
        if amount > 0:
            user.account += amount
            # transaction.commit()
            DBSession.add(user)
            message = 'اکانت با موقیت افزایش یافت'
        else:
            message = 'خطایی در افزایش حساب کاربری اتفاق افتاد'


    # for i in request.POST.items
    return {'message':message, 'user': user}


@view_config(route_name='score_exam_list', renderer='templates/score_exam_list.pt', permission='edit')
def score_exam_list(request):
    message = ''
    status = 2
    if 'submit.status' in request.POST:
        status = int(request.POST.get('status','2'))
    exams = DBSession.query(Exam).filter_by(status=status).all()
    return {'exams':exams}

@view_config(route_name='score_exam_page', renderer='templates/score_exam_page.pt', permission='edit')
def score_exam_page(request):
    message = ''
    exam_id = int(request.matchdict['exam_id'])
    exam = DBSession.query(Exam).filter_by(id=exam_id).first()
    true_answer = 0
    if 'submit' in request.POST:
        for a in exam.answers:

            a.score = int(request.POST.get(str(a.id), a.score))
            if a.score == 2:
                true_answer += 1


        if request.POST.get('scored', ''):

            print(true_answer, len(exam.answers), exam.course.min_score ,'===', true_answer / len(exam.answers), (exam.course.min_score / 100))
            if (true_answer / len(exam.answers) >= (exam.course.min_score / 100)) :
                exam.status = 4
            else:
                exam.status = 3

        DBSession.add(exam)
        transaction.commit()

    exam = DBSession.query(Exam).filter_by(id=exam_id).first()
        # for a in exam.answers:
        #     print(a.find_score(),a.score)


    return {'exam':exam}


